@@ -102,11 +102,11 @@ scrcpy.server_path = config['scrcpy-server']
 # ARGUMENT PARSER
 
 sys_argv = []
-for arg in range(len(sys.argv)):
+for arg in range(len(sys.argv)-1, -1, -1):
+    log("Scanning {}".format(sys.argv[arg]))
     if '.py' in sys.argv[arg] or 'guiscrcpy' in sys.argv[arg]:
-        sys_argv = sys.argv[arg:]
+        sys_argv = sys.argv[arg+1:]
         break
-sys.argv = sys_argv
 
 # interface adb if asked for
 if 'adb-interface' in sys.argv:
@@ -169,8 +169,8 @@ parser.add_argument(
 parser.add_argument(
     '--start-scrcpy-device-id',
     default='',
-    help="Provide the device id in the case of multiple devices. "
-         "(applicable only when '-s' or '--start' is passed)"
+    help="Provide the device id in the case of multiple devices."
+         " (applicable only when '-s' or '--start' is passed)"
 )
 parser.add_argument(
     '-r',
@@ -183,6 +183,22 @@ parser.add_argument(
     action='store_true',
     help="Interface guiscrcpy's mapper to guiscrcpy main executable"
 )
+parser.add_argument(
+    '--mapper-reset',
+    action='store_true',
+    help="Reset guiscrcpy's mapper configuration file"
+)
+parser.add_argument(
+    '--mapper-delay',
+    action='store_true',
+    help="Sets the delay before connecting and configuring mapper files"
+)
+parser.add_argument(
+    '--mapper-device-id',
+    default='',
+    help="Sets the device-id for mapper to configure (optional, "
+         "needed for multiple devices)"
+)
 parser.add_argument(
     '-w',
     '--disable-swipe',
@@ -200,6 +216,17 @@ parser.add_argument(
     action='store_true',
     help="Disable scrcpy processes (For debugging only)"
 )
+parser.add_argument(
+    '--panels-not-always-on-top',
+    action='store_true',
+    help="Remove the always on top default while running guiscrcpy"
+)
+parser.add_argument(
+    '--theme',
+    default='Breeze',
+    help="Set the default theme (based on PyQt5 themes - Fusion, Breeze, "
+         "Windows) (stored in configuration, override by --theme-no-cfg)"
+)
 parser.add_argument(
     '-f',
     '--force-window-frame',
@@ -231,8 +258,10 @@ parser.add_argument(
     action='store_true',
     help="Display guiscrcpy version"
 )
-args = parser.parse_args()
-
+log(f"Complete args {sys.argv}")
+log(f"Received {sys_argv}")
+args = parser.parse_args(sys_argv)
+log(f"Parsed {args}")
 # set argument debug level
 if args.debug:
     logging_priority = int(args.debug) * 10
@@ -240,27 +269,40 @@ else:
     logging_priority = 30
 logger = logging.Logger('guiscrcpy', logging_priority)
 
-# try using pynput, if exception handling not done here, it might fail in CI
-try:
-    from pynput import keyboard
-except Exception as e:
-    logger.warning("Running from tty, pass. E:{}".format(e))
-    keyboard = None
-
 logger.debug("Received flag {}".format(args.start))
 
 Header(VERSION)
 
 if args.version:
+    # Show version and exit lol
+    import inspect
+    from PyQt5 import Qt
+    _pyqt5_version = [
+        '%s = %s' % (k, v) for k, v in
+        vars(Qt).items() if
+        k.lower().find('version') >= 0 and not inspect.isbuiltin(v)
+    ]
+    print()
+    print("== PyQt5 Version ==")
+    print('\n'.join(sorted(_pyqt5_version)))
+    print()
+    if environment.system() == "Linux":
+        print("== CairoSVG version ==")
+        from cairosvg import VERSION as CAIRO_VERSION # noqa:
+        print("CairoSVG == {}".format(CAIRO_VERSION))
+        print()
+
     sys.exit(0)
 
 if args.reset:
+    # Resets the guiscrcpy.json configuration file
     cfgmgr.reset_config()
     print("Configuration files resetted successfully.")
 
 logger.debug("Current Working Directory {}".format(os.getcwd()))
 
 if args.connect:
+    # Connects to a device with a valid IP:PORT address
     adb_cnx_output = adb.command(adb.path, 'connect {}'.format(args.connect))
     print(
         adb_cnx_output.stdout.read().decode(),
@@ -268,6 +310,7 @@ if args.connect:
     )
 
 if args.start:
+    # Starts scrcpy before calling guiscrcpy UI with limited configuration
     devices = adb.devices_detailed(adb.path)
     logger.debug("RUNNING SCRCPY DIRECTLY")
     scrcpy_args = ""
@@ -284,6 +327,32 @@ if args.start:
         scrcpy_args += " --turn-screen-off "
     scrcpy.start(scrcpy.path, scrcpy_args)
 
+if args.mapper_reset:
+    # Reset the mapper configuration if mapper is called.
+    mapper_cfg_path = os.path.join(
+        cfgmgr.get_cfgpath(), 'guiscrcpy.mapper.json'
+    )
+    # A ternary version of removing a file if it exists
+    # https://stackoverflow.com/questions/10840533/
+    # most-pythonic-way-to-delete-a-file-which-may-not-exist
+    mapper_configuration_file_exists = os.remove(mapper_cfg_path) if \
+        os.path.exists(
+        mapper_cfg_path) else None
+    if mapper_configuration_file_exists:
+        print("guiscrcpy mapper configuration file has been removed.")
+        print("Removed {}".format(mapper_configuration_file_exists))
+        sys.exit(0)
+    else:
+        print("guiscrcpy mapper configuration is not created yet.")
+        sys.exit(1)
+
+if args.mapper:
+    from guiscrcpy.lib import mapper
+    # Initialize the mapper if it is called.
+    print('Initializing guiscrcpy mapper v3.5-')
+    mapper.file_check()
+    sys.exit(0)
+
 logger.debug("Importing modules...")
 
 
@@ -354,10 +423,10 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             self.fullscreen.setChecked(False)
         if is_running("scrcpy"):
             logger.debug("SCRCPY RUNNING")
-            self.private_message_box_adb.setText("SCRCPY SERVER RUNNING")
+            self.display_public_message("SCRCPY SERVER RUNNING")
         else:
             logger.debug("SCRCPY SERVER IS INACTIVE")
-            self.private_message_box_adb.setText("SCRCPY SERVER NOT RUNNING")
+            self.display_public_message("SCRCPY SERVER NOT RUNNING")
 
         # CONNECT DIMENSION CHECK BOX TO STATE CHANGE
         self.dimensionDefaultCheckbox.stateChanged.connect(
@@ -374,11 +443,8 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         try:
             if config['extra']:
                 self.flaglineedit.setText(config['extra'])
-            else:
-                pass
         except Exception as err:
             logger.debug(f"Exception: flaglineedit.text(config[extra]) {err}")
-            pass
 
         # set swipe instance, bottom instance and
         # side instance as enabled by default
@@ -397,14 +463,22 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         self.mapnow.clicked.connect(self.bootstrap_mapper)
         self.network_button.clicked.connect(self.network_mgr)
         self.settings_button.clicked.connect(self.settings_mgr)
+        # self.devices_view.itemChanged.connect(self.update_rotation_combo_cb)
+        self.devices_view.itemClicked.connect(self.update_rotation_combo_cb)
         self.refreshdevices.clicked.connect(
             self.scan_devices_update_list_view
         )
+        self.restart_adb_server.clicked.connect(
+            self.restart_adb_server_guiscrcpy)
         self.devices_view.itemClicked.connect(self.more_options_device_view)
         self.devices_view.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         self.scan_config_devices_update_list_view()
         self.refresh_devices()
 
+    @staticmethod
+    def restart_adb_server_guiscrcpy():
+        adb.kill_adb_server(adb.path)
+
     def refresh_devices(self):
         """
         A slot for refreshing the QListView
@@ -418,7 +492,18 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         change in the QListBox
         :return:
         """
-        raise NotImplementedError("Maybe try waiting for me to finish it")
+        if self.devices_view.currentItem():
+            _, device_id = self.current_device_identifier()
+            _rotation = config['device']\
+                .get(device_id, dict()).get(
+                    'rotation',
+                    self.device_rotation.currentIndex()
+            )
+        else:
+            _rotation = config.get(
+                "rotation", self.device_rotation.currentIndex()
+            )
+        self.device_rotation.setCurrentIndex(_rotation)
 
     def settings_mgr(self):
         from guiscrcpy.ux.settings import InterfaceSettings
@@ -440,10 +525,11 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             mapper.file_check()
         else:
             logger.warning(
-                "guiscrcpy ~ mapper is not initialized. "
-                "Initialize by running" +
-                "$ guiscrcpy-mapper" + "reset points by" +
-                "$ guiscrcpy-mapper -r"
+                "guiscrcpy ~ mapper is not initialized. \n"
+                "Initialize by running \n\n"
+                "$ guiscrcpy --mapper \n\n"
+                "reset points by \n\n" +
+                "$ guiscrcpy --mapper --mapper-reset\n"
             )
 
     @staticmethod
@@ -522,10 +608,11 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         if 'Disconnect' in button.text():
             menu = QMenu("Menu", self)
             menu.addAction("Pair / Ping", self.ping_paired_device)
-            menu.addAction("Attempt TCPIP on device", self.ping_paired_device)
+            menu.addAction("Attempt TCPIP on device", self.tcpip_paired_device)
             menu.addAction("Forget device", self.forget_paired_device)
         else:
             menu = QMenu("Menu", self)
+            menu.addAction("Attempt TCPIP on device", self.tcpip_paired_device)
             menu.addAction("Attempt reconnection", self.ping_paired_device)
             menu.addAction("Refresh", self.refresh_devices)
         _, identifier = self.current_device_identifier()
@@ -558,7 +645,10 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         # get device specific configuration
         model, identifier = self.current_device_identifier()
         picture_file_path = cfgmgr.get_cfgpath()
-        sha = hashlib.sha256(str(identifier).encode()).hexdigest()[5:5+6]
+        __sha_shift = config.get('sha_shift', 5)
+        sha = hashlib.sha256(
+            str(identifier).encode()
+        ).hexdigest()[__sha_shift:__sha_shift + 6]
         log(f"Creating desktop shortcut sha: {sha}")
         path_to_image = os.path.join(picture_file_path, identifier+'.png')
         svg2png(
@@ -573,7 +663,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                 break
         else:
             aend = None
-            pass
+
         sys_args_desktop = sys.argv[:aend]
 
         # check if its a python file
@@ -610,57 +700,79 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         # announce it to developers / users
         log(f"Path to desktop file : {path_to_desktop_file}")
         print("Desktop file generated successfully")
-        self.private_message_box_adb.setText("Desktop file has been created")
+        self.display_public_message("Desktop file has been created")
 
-    def ping_paired_device(self):
+    def is_connection_success_handler(self, output: Popen, ip=None):
+        out, err = output.communicate()
+        if 'failed' in out.decode() or 'failed' in err.decode():
+            self.display_public_message(
+                "Failed to connect to {}. See the logs for more "
+                "information".format(ip)
+            )
+            print("adb:", out.decode(), err.decode())
+        else:
+            self.display_public_message(
+                "Connection command completed successfully"
+            )
+
+    def ping_paired_device(self, device_id=None):
         # update the configuration file first
-        _, identifier = self.current_device_identifier()
-        if identifier.count('.') == 3:
-            wifi_device = True
-        else:
-            wifi_device = False
-        try:
-            config['device'][identifier]['wifi'] = wifi_device
-        except KeyError:
-            log(f"Failed writing the configuration 'wifi' key to {identifier}")
-
-        if wifi_device:
-            ip = self.current_device_identifier()[1]
-            output = adb.command(adb.path, 'connect {}'.format(ip))
-            out, err = output.communicate()
-            if 'failed' in out.decode() or 'failed' in err.decode():
-                self.private_message_box_adb.setText(
-                    "Failed to connect to {}. See the logs for more "
-                    "information".format(ip)
-                )
-                print("adb:", out.decode(), err.decode())
+        if not device_id:
+            _, identifier = self.current_device_identifier()
+            if identifier.count('.') == 3:
+                wifi_device = True
             else:
-                self.private_message_box_adb.setText(
-                    "Connection command completed successfully"
-                )
+                wifi_device = False
+            try:
+                config['device'][identifier]['wifi'] = wifi_device
+            except KeyError:
+                log(f"Failed writing the configuration "
+                    f"'wifi' key to {identifier}")
+
+            if wifi_device:
+                ip = self.current_device_identifier()[1]
+                output = adb.command(adb.path, 'connect {}'.format(ip))
+                self.is_connection_success_handler(output, ip=ip)
+            else:
+                adb.command(adb.path, 'reconnect offline')
+            # As we have attempted to connect; refresh the panel
+            self.refresh_devices()
         else:
-            adb.command(adb.path, 'reconnect offline')
-        # As we have attempted to connect; refresh the panel
-        self.refresh_devices()
+            output = adb.command(adb.path, 'connect {}'.format(device_id))
+            self.is_connection_success_handler(output, ip=device_id)
 
     def tcpip_paired_device(self):
-        ecode = adb.tcpip(adb.path)
-        if ecode != 0:
-            self.private_message_box_adb.setText(
+        if self.devices_view.currentItem():
+            _, identifier = self.current_device_identifier()
+        else:
+            identifier = ""
+        __exit_code = adb.tcpip(adb.path, identifier=identifier)
+        if __exit_code != 0:
+            self.display_public_message(
                 "TCP/IP failed on device. "
                 "Please reconnect USB and try again"
             )
         else:
-            self.private_message_box_adb.setText(
+            self.display_public_message(
                 "TCP/IP completed successfully."
             )
-            self.ping_paired_device()
+            time.sleep(0.1)  # wait for everything to get settled
+            if identifier.count('.') >= 3:
+                self.ping_paired_device(device_id=identifier)
+            else:
+                self.ping_paired_device()
 
-    def current_device_identifier(self):
+    def current_device_identifier(self, need_status=False):
         if self.devices_view.currentItem():
-            return \
-                self.devices_view.currentItem().text().split()[0], \
-                self.devices_view.currentItem().text().split()[1]
+            if need_status:
+                return \
+                    self.devices_view.currentItem().text().split()[0], \
+                    self.devices_view.currentItem().text().split()[1], \
+                    self.devices_view.currentItem().text().split()[2]
+            else:
+                return \
+                    self.devices_view.currentItem().text().split()[0], \
+                    self.devices_view.currentItem().text().split()[1]
         else:
             raise ValueError("No item is selected in QListView")
 
@@ -682,13 +794,16 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                             status='Disconnected'
                         )
                     )
-
+                __sha_shift = config.get('sha_shift', 5)
+                __sha = hashlib.sha256(
+                    str(i).encode()).hexdigest()[__sha_shift:__sha_shift+6]
                 devices_view_list_item.setToolTip(
-                    "Device: {d}\n"
+                    "<span style='color: #{color}'>Device</snap>: <b>{d}</b>\n"
                     "Status: {s}".format(
                         d=i,
                         s="Disconnected. Right click 'ping' to attempt "
                           "reconnect",
+                        color=__sha
                     )
                 )
                 devices_view_list_item.setFont(QFont('Noto Sans', pointSize=8))
@@ -701,6 +816,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         :return:
         """
         # self.devices_view.clear()
+        device_exists_in_view = False
         paired_devices = []
         for index in range(self.devices_view.count()):
             paired_devices.append(self.devices_view.item(index))
@@ -734,6 +850,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                 icon = ':/icons/icons/portrait_mobile_warning.svg'
 
             if i['status'] == 'no_permission':
+                log("pairfilter: 5")
                 # https://stackoverflow.com/questions/
                 # 53887322/adb-devices-no-permissions-user-in-
                 # plugdev-group-are-your-udev-rules-wrong
@@ -741,12 +858,13 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                     " incorrect. See https://stackoverflow.com/questions"\
                     "/53887322/adb-devices-no-permissions-user-in-plugdev-"\
                     "group-are-your-udev-rules-wrong"
-                self.private_message_box_adb.setText(udev_error)
+                self.display_public_message(udev_error)
                 print(udev_error)
                 return []
             # Check if device is unauthorized
             elif i['status'] == "unauthorized":
                 log("unauthorized device detected: Click Allow on your device")
+                log("pairfilter: 4")
                 # The device is connected; and might/might't paired in the past
                 # And is connected to the same IP address
                 # It is possibly a bug with the connection;
@@ -754,7 +872,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                 # device with the error
                 paired = False
                 device_paired_and_exists = False
-                self.private_message_box_adb.setText(
+                self.display_public_message(
                     f"{i['identifier']} is unauthorized. Please click allow "
                     f"on your device."
                 )
@@ -775,6 +893,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                 else:
                     for paired_device in paired_devices:
                         if paired_device.text().split()[0] == i['model']:
+                            log("pairfilter: 1")
                             paired = True
                             devices_view_list_item = paired_device
                             # as we have found a paired device
@@ -791,11 +910,16 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                             break
                         elif paired_device.text().split()[1] ==\
                                 i['identifier']:
-
+                            log("pairfilter: 2")
+                            self.remove_device_device_view(
+                                i['identifier'],
+                                statuses=['offline', 'unauthorized']
+                            )
                             devices_view_list_item = QListWidgetItem()
                             paired = False
                             break
                     else:
+                        log("pairfilter: 3")
                         paired = False
                         devices_view_list_item = QListWidgetItem()
 
@@ -808,19 +932,28 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                     status=i['status']
                 )
             )
+            __sha_shift = config.get('sha_shift', 5)
+            __sha = hashlib.sha256(
+                str(i['identifier']).encode()
+            ).hexdigest()[__sha_shift:__sha_shift + 6]
             devices_view_list_item.setToolTip(
-                "Device: {d}\n"
-                "Model: {m}\n"
-                "Alias: {a}\n"
-                "Status: {s}\n"
-                "Transport ID: {t}\n"
+                "Device: "
+                "<span style='color: #{inv_color};background-color: #{color}'>"
+                "<b>{d}</b></span>\n"
+                "<br>"
+                "Model: {m}\n<br>"
+                "Alias: {a}\n<br>"
+                "Status: {s}\n<br>"
+                "Transport ID: {t}\n<br>"
                 "Paired: {p}".format(
                     d=i['identifier'],
                     m=i['model'],
                     a=i['product'],
                     s=i['status'],
                     t=i['transport_id'],
-                    p=paired
+                    p=paired,
+                    color=__sha,
+                    inv_color=str(hex(0xFFFFFF - int(__sha, 16))[2:])
                 )
             )
 
@@ -830,6 +963,10 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
                 # we need to only neglect wifi devices
                 # paired usb device need to still show in the display
                 continue
+            elif device_exists_in_view:
+                # devices exists in the list with the same status and
+                # we should not add the new detected list item
+                continue
             # If and only if the device doesn't exist; add it
             self.devices_view.addItem(devices_view_list_item)
         return __devices
@@ -844,10 +981,12 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         """
         for index in range(self.devices_view.count() - 1, -1, -1):
             for status in statuses:
-                if str(identifier) in self.devices_view.item(index).text() \
-                        and \
-                        str(status) in self.devices_view.item(index).text():
-                    self.devices_view.takeItem(index)
+                if self.devices_view.item(index):
+                    if str(identifier) in self.devices_view.item(index).text()\
+                            and \
+                            str(status) in \
+                            self.devices_view.item(index).text():
+                        self.devices_view.takeItem(index)
         return
 
     def __dimension_change_cb(self):
@@ -869,12 +1008,10 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
     def __slider_change_cb(self):
         config['dimension'] = int(self.dimensionSlider.value())
         self.dimensionText.setText(str(config['dimension']) + "px")
-        pass
 
     def __dial_change_cb(self):
         config['bitrate'] = int(self.dial.value())
         self.bitrateText.setText(str(config['bitrate']) + "KB/s")
-        pass
 
     def progress(self, val):
         self.progressBar.setValue(val)
@@ -883,13 +1020,19 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         else:
             return val + 100 / 20
 
-    def start_act(self):
-        # prepare launch of scrcpy,
-        # reset colors
-        # reset vars
+    @staticmethod
+    def is_device_unusable(status):
+        if any(('unauth' in status, 'offline' in status)):
+            return True
+        else:
+            return False
 
-        # 1: reset
-        progress = self.progress(0)
+    def show_device_status_failure(self, status):
+        self.display_public_message(
+            f"Device is {status}. Please reconnect / press allow."
+        )
+
+    def __reset_message_box_stylesheet(self):
         stylesheet = \
             "background-color: qlineargradient(" \
             "spread:pad, x1:0, y1:0, x2:1, y2:1, " \
@@ -898,9 +1041,36 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             "border-radius: 10px;"
         self.private_message_box_adb.setStyleSheet(stylesheet)
 
+    def __select_first_device(self):
+        self.devices_view.setCurrentIndex(
+            QModelIndex(self.devices_view.model().index(0, 0))
+        )
+
+    def display_public_message(self, message):
+        """
+        sets the message box information (GUI) with the string message
+        :param message: str
+        :return: None
+        """
+        self.private_message_box_adb.setText(message)
+
+    def start_act(self):
+        """
+        Main brain of guiscrcpy; handles what to do when
+        :return:
+        """
+        # prepare launch of scrcpy,
+        # reset colors
+        # reset vars
+
+        # 1: reset
+        self.options = ""
+        progress = self.progress(0)
+        self.__reset_message_box_stylesheet()
+
         # ====================================================================
         # 2: Update UI to start checking
-        self.private_message_box_adb.setText("CHECKING DEVICE CONNECTION")
+        self.display_public_message("CHECKING DEVICE CONNECTION")
         initial_time = time.time()
         progress = self.progress(progress)
 
@@ -908,33 +1078,50 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         # 3: Check devices
         values_devices_list = self.scan_devices_update_list_view()
         if len(values_devices_list) == 0:
-            self.private_message_box_adb.setText("Could not find any devices")
+            # Could not detect any device
+            self.display_public_message("Could not find any devices")
             return 0
         elif self.devices_view.currentIndex() is None and \
                 len(values_devices_list) != 1:
-            self.private_message_box_adb.setText(
-                "Please select a device below."
-            )
+            # No device is selected and more than one device found
+            self.display_public_message("Please select a device below.")
             return 0
         else:
+            # Device selected
+            log("DEVICE LIST", len(values_devices_list), values_devices_list)
             if len(values_devices_list) == 1:
-                self.devices_view.setCurrentIndex(
-                    QModelIndex(self.devices_view.model().index(0, 0))
-                )
+                # found only one device
+                # ======================================================
+                # Store the current rotation temporarily
+                __selected_rotation = self.device_rotation.currentIndex()
+                self.__select_first_device()
+                # Restore the selected rotation
+                self.device_rotation.setCurrentIndex(__selected_rotation)
+                # =======================================================
+                # get the status and identifier of the device;
+                # return if device is not in a connectable state
                 try:
-                    _, device_id = self.current_device_identifier()
+                    _, device_id, _stat = \
+                        self.current_device_identifier(need_status=True)
+                    if self.is_device_unusable(_stat):
+                        self.show_device_status_failure(_stat)
+                        return 0
                 except ValueError:
-                    self.private_message_box_adb.setText(
+                    self.display_public_message(
                         "Please select a device from the list view"
                     )
                     return 0
                 more_devices = False
             elif self.devices_view.currentItem() is None:
-                self.private_message_box_adb.setText("Please select a device "
-                                                     "below.")
+                # no item is selected
+                self.display_public_message("Please select a device below.")
                 return 0
             else:
-                _, device_id = self.current_device_identifier()
+                _, device_id, _stat = self.current_device_identifier(
+                    need_status=True)
+                if self.is_device_unusable(_stat):
+                    self.show_device_status_failure(_stat)
+                    return 0
                 log("Device_id = {}".format(device_id))
                 more_devices = True
         progress = self.progress(progress)
@@ -946,7 +1133,6 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             self.dimensionSlider.setEnabled(False)
             self.dimensionText.setText("DEFAULT")
             config['dimension'] = None
-
         else:
             self.dimensionSlider.setEnabled(True)
             config['dimension'] = int(self.dimensionSlider.value())
@@ -955,7 +1141,6 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
         # edit configuration files to update dimension key
         if config['dimension'] is None:
             self.options = " "
-            pass
         elif config['dimension'] is not None:
             self.options = " -m " + str(config['dimension'])
         else:
@@ -1010,7 +1195,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             '{str(self.bitrateText.text().split()[1][0])}'. " \
                                                f"Please use only K, M, T only"
             print(multiplier_error)
-            self.private_message_box_adb.setText(multiplier_error)
+            self.display_public_message(multiplier_error)
             return False
         if self.bitrateText.text().split()[0].isdigit():
             bitrate_integer = int(self.bitrateText.text().split()[0])
@@ -1032,9 +1217,14 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
 
         # ====================================================================
         # 11: Initialize User Experience Mapper
-        ux = UXMapper(device_id=device_id)
+        ux = UXMapper(
+            device_id=device_id,
+            sha_shift=config.get('sha_shift', 5)
+        )
         progress = self.progress(progress)
-
+        always_on_top = \
+            config.get('panels_always_on_top', False) or \
+            not args.panels_not_always_on_top
         # ====================================================================
         # 12: Init side_panel if necessary
         if self.check_side_panel.isChecked():
@@ -1042,11 +1232,13 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             side_instance = InterfaceToolkit(
                 parent=self,
                 ux_mapper=ux,
-                frame=args.force_window_frame
+                frame=args.force_window_frame,
+                always_on_top=always_on_top
             )
             for instance in self.child_windows:
                 if instance.ux.get_sha() == side_instance.ux.get_sha() and \
-                        instance.name == side_instance.name:
+                        instance.name == side_instance.name and \
+                        not instance.isHidden():
                     break
             else:
                 side_instance.init()
@@ -1062,11 +1254,13 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             panel_instance = Panel(
                 parent=self,
                 ux_mapper=ux,
-                frame=args.force_window_frame
+                frame=args.force_window_frame,
+                always_on_top=always_on_top
             )
             for instance in self.child_windows:
                 if instance.ux.get_sha() == panel_instance.ux.get_sha() and \
-                        instance.name == panel_instance.name:
+                        instance.name == panel_instance.name and \
+                        not instance.isHidden():
                     break
             else:
                 panel_instance.init()
@@ -1081,11 +1275,13 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             config['panels']['swipe'] = True
             swipe_instance = SwipeUX(
                 ux_wrapper=ux,
-                frame=args.force_window_frame
+                frame=args.force_window_frame,
+                always_on_top=always_on_top
             )  # Load swipe UI
             for instance in self.child_windows:
                 if instance.ux.get_sha() == swipe_instance.ux.get_sha() and \
-                        instance.name == swipe_instance.name:
+                        instance.name == swipe_instance.name and \
+                        not instance.isHidden():
                     break
             else:
                 swipe_instance.init()
@@ -1145,7 +1341,7 @@ class InterfaceGuiscrcpy(QMainWindow, Ui_MainWindow):
             # device is found
             arguments_scrcpy = f"-s {device_id} {arguments_scrcpy}"
             # tell end users that the color of the device is this
-            self.private_message_box_adb.setText(
+            self.display_public_message(
                 f"Device {device_id} is connected; (color id matches "
                 f"toolkit color)"
             )
@@ -1183,11 +1379,15 @@ def bootstrap0():
     # enable High DPI scaling
     QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
     QApplication.setAttribute(
-        QtCore.Qt.AA_UseHighDpiPixmaps, True)  # use HIGH DPI icons
+        QtCore.Qt.AA_UseHighDpiPixmaps, True
+    )  # use HIGH DPI icons
     app = QtWidgets.QApplication(sys.argv)
 
-    app.setStyle('Breeze')
-    app.setStyleSheet(dark_stylesheet())
+    app.setStyle(args.theme)
+
+    if args.theme == 'Breeze':
+        # The Qdarkstylesheet is based on Breeze, lets load them on default
+        app.setStyleSheet(dark_stylesheet())
 
     splash_pix = QPixmap(":/res/ui/guiscrcpy-branding.png")
     splash = QtWidgets.QSplashScreen(splash_pix)
